@@ -4,67 +4,86 @@ use strict;
 use warnings;
 use IO::Select;
 use File::Spec::Functions;
+use Time::HiRes qw( sleep );
+use File::Which qw( where );
+use Env qw( @PATH $PROGRAMFILES );
+use File::DosGlob qw( glob );
+
 
-our $VERSION = '0.09';
 our $HOLD_PIPE_X;
-
-my $this;
+our $IS_WIN = ($^O =~ m/^(?:.*?win32|dos)$/i) ? 1 : 0;
 
 
 sub new {
-    my $class = shift;
-
-    if( !defined $this ) {
-        $this = bless( {}, $class );
-
-        my $method = ( $^O =~ /^(?:.*?win32|dos)$/i ) ? 'Win32' : 'Linux';
-        $this->$method( @_ );
-
-        return unless $this->{ OS };
-    }
-
+    my ($class, %args) = @_;
+    my $this = {};
+    bless $this, ref($class) || $class;
+    $this->initialize( %args );
     return $this;
 }
 
 
-sub pipe {
-    my( $this, %args ) = @_;
-
-    $this->{ LOG_DIR } = $args{ log_dir } || catfile($this->{TMP_DIR}, 'Statistics-R');
-
+sub initialize {
+    my ($this, %args) = @_;
+    
+    # Find and create log dir
+    $this->{ LOG_DIR } = $args{ log_dir } || catfile( File::Spec->tmpdir(), 'Statistics-R');
+    # Fix by CTB for RT Bug #17956: log_dir should be in tmp_dir by default
     if ( !-e $this->{ LOG_DIR } ) { mkdir( $this->{ LOG_DIR }, 0777 ); }
-
     if (   !-d $this->{ LOG_DIR }
         || !-r $this->{ LOG_DIR }
         || !-w $this->{ LOG_DIR } )
     {
         die "Error: Could not read or write the LOG_DIR directory ".$this->{LOG_DIR}."\n"
     }
-
-    $this->{ OUTPUT_DIR } = $args{ output_dir } || catfile($this->{LOG_DIR}, 'output');
-
-    if ( !-d $this->{ OUTPUT_DIR } || !-e $this->{ OUTPUT_DIR } ) {
-        mkdir( $this->{ OUTPUT_DIR }, 0777 );
-    }
-
-    if ( !-r $this->{ OUTPUT_DIR } || !-w $this->{ OUTPUT_DIR } ) {
-        die "Error: Could not read or write the OUTPUT_DIR directory ".$this->{OUTPUT_DIR}."\n"
+    
+    # Find full path of R binary
+    if ( $args{ r_bin } || $args{ R_bin } ) {
+        # User-specified location
+        $this->{ R_BIN } =  $args{ r_bin } || $args{ R_bin };
+        if ( !-s $this->{ R_BIN } ) {
+            die "Error: Could not find the R binary at the specified location, ".
+                $this->{ R_BIN }."\n";
+        }
+    } else {
+        # Windows-specific PATH adjustement
+        win32_path_adjust() if $IS_WIN;
+        # Locate R in PATH using File::Which
+        my @paths = where('R');
+        if (scalar @paths == 0) {
+            die "Error: Could not find the R binary! Make sure it is in your ".
+                "PATH environment variable or specify its location using the ".
+                "r_bin option of the new() method.\n";
+        }
+        $this->{ R_BIN } = $paths[0];
+        #if (scalar @paths > 1) {
+        #    warn "Warning: Multiple binaries where found for R. Using the first".
+        #         " one... ".$this->{ R_BIN }."\n";  
+        #}
     }
+    $this->{ R_BIN } = win32_space_quote( $this->{ R_BIN } ) if $IS_WIN;
+    $this->{ START_CMD } = "$this->{R_BIN} --slave --vanilla ";
 
+    # Files necessary for piping to and from R
     $this->{ START_R }    = catfile($this->{LOG_DIR}, 'start.r');
     $this->{ OUTPUT_R }   = catfile($this->{LOG_DIR}, 'output.log');
     $this->{ PROCESS_R }  = catfile($this->{LOG_DIR}, 'process.log');
     $this->{ PID_R }      = catfile($this->{LOG_DIR}, 'R.pid');
-    $this->{ LOCK_R }     = catfile($this->{LOG_DIR}, 'lock.pid');
     $this->{ STARTING_R } = catfile($this->{LOG_DIR}, 'R.starting');
-    $this->{ STOPING_R }  = catfile($this->{LOG_DIR}, 'R.stoping');
+    $this->{ STOPPING_R } = catfile($this->{LOG_DIR}, 'R.stopping');
+    $this->{ LOCK_R }     = catfile($this->{LOG_DIR}, 'lock.pid');
+    
+    # The name of R input file will be something like input.1.r
+    $this->{ INPUT_R_PREFIX } = catfile($this->{LOG_DIR}, 'input.');
+    $this->{ INPUT_R_SUFFIX } = '.r';
 
+    return 1;
 }
 
 
 sub bin {
     my $this = shift;
-    $this->{ R_BIN };
+    return $this->{ R_BIN };
 }
 
 
@@ -74,55 +93,63 @@ sub send {
     $cmd =~ s/\r\n?/\n/gs;
     $cmd .= "\n" if $cmd !~ /\n$/;
     $cmd =~ s/\n/\r\n/gs;
-
+    
     while ( $this->is_locked ) { sleep( 1 ); }
-
+    
     my $n = $this->read_processR || 0;
     $n = 1 if $n eq '0' || $n eq '';
-
+    
+    # Increment file number until the first free slot
     my $file = catfile( $this->{LOG_DIR}, "input.$n.r" );
-
     while ( -e $file || -e "$file._" ) {
         ++$n;
         $file = catfile( $this->{LOG_DIR}, "input.$n.r" );
     }
-
-    open( my $fh, ">$file._" );
-    print $fh "$cmd\n";
-    close( $fh );
-
+    
+    $this->write_file( "$file._", "$cmd\n");
+    
     chmod( 0777, "$file._" );
 
     $this->{ OUTPUT_R_POS } = -s $this->{ OUTPUT_R };
 
     rename( "$file._", $file );
-
+    
     my $has_quit = 1 if $cmd =~ /^\s*(?:q|quit)\s*\(.*?\)\s*$/s;
 
     ##print "CMD[$n]$has_quit>> $cmd\n" ;
 
     my $status = 1;
     my $delay  = 0.02;
-
+    
     my ( $x, $xx );
-    while (( !$has_quit || $this->{ STOPING } == 1 )
-        && -e $file
-        && $this->is_started( !$this->{ STOPING } ) )
+    while (
+            ( !$has_quit || $this->{ STOPPING } == 1 )
+            &&  -e $file
+            &&  $this->is_started( !$this->{ STOPPING } )
+          )
     {
+        
         ++$x;
         ##print "sleep $file\n" ;
-        select( undef, undef, undef, $delay );
+        sleep( $delay );
         if ( $x == 20 ) {
+            
             my ( undef, $data ) = $this->read_processR;
-            if ( $data =~ /\s$n\s+\.\.\.\s+\// ) { last; }
+            
+            if ( $data =~ /\s$n\s+\.\.\.\s+\// ) { 
+                last;
+            }
             $x = 0;
             ++$xx;
             $delay = 0.5;
         }
-        if ( $xx || 0 > 5 ) { $status = undef; }    ## xx > 5 = x > 50
+        if ( $xx || 0 > 5 ) {
+            
+            $status = undef;
+        }    ## xx > 5 = x > 50
     }
-
-    if ( $has_quit && !$this->{ STOPING } ) { $this->stop( 1 ); }
+    
+    if ( $has_quit && !$this->{ STOPPING } ) { $this->stop( 1 ); }
 
     return $status;
 }
@@ -133,7 +160,7 @@ sub receive {
 
     $timeout = -1 if !defined $timeout;
 
-    open( my $fh, $this->{ OUTPUT_R } );
+    open( my $fh, '<', $this->{ OUTPUT_R } ) or die "Error: Could not read file ".$this->{ OUTPUT_R }."\n$!\n";
     binmode( $fh );
     seek( $fh, ( $this->{ OUTPUT_R_POS } || 0 ), 0 );
 
@@ -162,17 +189,6 @@ sub receive {
 *read = \&receive;
 
 
-sub clean_log_dir {
-    my $this = shift;
-
-    my @dir = $this->cat_dir( $this->{ LOG_DIR }, 0, 0, 1 );
-    foreach my $dir_i ( @dir ) {
-        ##print "RM>> $dir_i\n" ;
-        unlink $dir_i if $dir_i !~ /R\.(?:starting|stoping)$/;
-    }
-}
-
-
 sub is_started {
     my( $this, $can_auto_start ) = @_;
 
@@ -187,14 +203,13 @@ sub is_started {
         $this->sleep_unsync;
 
         if ( $can_auto_start ) {
-            $this->wait_stoping;
+            $this->wait_stopping;
             if ( -e $this->{ PID_R } ) { $this->wait_starting; }
         }
 
         if ( -s $this->{ PID_R } ) { $this->update_pid; }
         elsif ( $can_auto_start ) {
-            open( my $fh, ">$this->{PID_R}" );
-            close( $fh );
+            $this->touch_file( $this->{PID_R} );
             chmod( 0777, $this->{ PID_R } );
             $this->stop;
             return $this->start_shared;
@@ -209,15 +224,15 @@ sub is_started {
 sub lock {
     my $this = shift;
 
-    while ( $this->is_locked ) { select( undef, undef, undef, 0.5 ); }
+    while ( $this->is_locked ) { sleep( 0.5 ); }
 
-    open( my $fh, ">$this->{LOCK_R}" );
-    print $fh "$$\n";
-    close( $fh );
+    $this->write_file( $this->{LOCK_R}, "$$\n");
 
     chmod( 0777, $this->{ LOCK_R } );
 
     $this->{ LOCK } = 1;
+    
+    return 1;
 }
 
 
@@ -238,10 +253,7 @@ sub is_locked {
         || !-e $this->{ LOCK_R }
         || !-s $this->{ LOCK_R } );
 
-    open( my $fh, $this->{ LOCK_R } );
-    my $pid = join '', <$fh>;
-    close( $fh );
-    $pid =~ s/\s//gs;
+    my $pid = $this->read_file( $this->{ LOCK_R } );
 
     return undef if $pid == $$;
 
@@ -258,6 +270,7 @@ sub update_pid {
     my $pid = join '', <$fh>;
     close( $fh );
     $pid =~ s/\s//gs;
+    
     return $this->{ PID } if $pid eq '';
     $this->{ PID } = $pid;
 }
@@ -268,43 +281,36 @@ sub chmod_all {
 
     chmod( 0777,
         $this->{ LOG_DIR },
-        $this->{ OUTPUT_DIR },
         $this->{ START_R },
         $this->{ OUTPUT_R },
         $this->{ PROCESS_R },
         $this->{ LOCK_R },
         $this->{ PID_R } );
+        
 }
 
 
 sub start {
     my $this = shift;
-
+    
     return if $this->is_started;
 
     $this->stop( undef, undef, 1 );
-
+    
     $this->clean_log_dir;
-
+    
     $this->save_file_startR;
 
-    my $fh;
-
-    open $fh, '>', $this->{PID_R};
-    close $fh;
-    open $fh, '>', $this->{OUTPUT_R};
-    close $fh;
-    open $fh, '>', $this->{PROCESS_R};
-    close $fh;
-
+    $this->touch_file( $this->{PID_R} );
+    $this->touch_file( $this->{OUTPUT_R} );
+    $this->touch_file( $this->{PROCESS_R} );
+    
     $this->chmod_all;
 
-    my $cmd = $this->{START_CMD}." <start.r >output.log";
-
-    chdir $this->{LOG_DIR};
-    my $pid = open( my $read, "| $cmd" );
-    return if !$pid;
+    my $cmd = $this->{START_CMD}." < ".$this->{START_R}." > ".$this->{ OUTPUT_R };
 
+    my $pid = open( my $read, "| $cmd" ) or die "Error: Could not run the commmand\n$!\n";
+    
     $this->{ PIPE } = $read;
 
     $this->{ HOLD_PIPE_X } = ++$HOLD_PIPE_X;
@@ -316,17 +322,21 @@ sub start {
     $this->{ PID } = $pid;
 
     $this->chmod_all;
-
-    while ( !-s $this->{ PID_R } ) { select( undef, undef, undef, 0.05 ); }
-
-    $this->update_pid;
-
-    while ( $this->read_processR() eq '' ) {
-        select( undef, undef, undef, 0.10 );
+    
+    # Wait
+    while ( !-s $this->{ PID_R } ) {
+        sleep( 0.05 );
     }
-
+    
+    $this->update_pid;
+    
+    # Wait
+    while ( $this->read_processR() eq '' ) {
+        sleep( 0.10 );
+    }
+    
     $this->chmod_all;
-
+    
     return 1;
 }
 
@@ -359,11 +369,11 @@ sub wait_starting {
 }
 
 
-sub wait_stoping {
+sub wait_stopping {
     my $this = shift;
 
     my $c;
-    while ( -e $this->{ STOPING_R } ) {
+    while ( -e $this->{ STOPPING_R } ) {
         ++$c;
         sleep( 1 );
         if ( $c == 10 ) { return; }
@@ -376,7 +386,8 @@ sub sleep_unsync {
     my $this = shift;
 
     my $n = "0." . int( rand( 100 ) );
-    select( undef, undef, undef, $n );
+    sleep( $n );
+    return 1;
 }
 
 
@@ -386,7 +397,7 @@ sub start_shared {
     return if $this->is_started;
     $this->{ START_SHARED } = 1;
 
-    $this->wait_stoping;
+    $this->wait_stopping;
 
     $this->wait_starting;
 
@@ -399,13 +410,13 @@ sub start_shared {
 
         $this->update_pid;
         $this->{ OUTPUT_R_POS } = 0;
-        $this->{ STOPING }      = undef;
+        $this->{ STOPPING }      = undef;
 
         $this->chmod_all;
 
         if ( $this->is_started ) {
             while ( $this->read_processR() eq '' ) {
-                select( undef, undef, undef, 0.10 );
+                sleep( 0.10 );
             }
             return 1;
         }
@@ -413,8 +424,7 @@ sub start_shared {
 
     my $starting;
     if ( !-e $this->{ STARTING_R } ) {
-        open( my $fh, ">$this->{STARTING_R}" );
-        close( $fh );
+        $this->touch_file( $this->{STARTING_R} );
         $starting = 1;
     }
 
@@ -439,18 +449,14 @@ sub start_shared {
 
 sub stop {
     no warnings; # squash "Killed" warning for now
-    my $this = shift;
-    my $no_send         = shift( @_ );
-    my $not_started     = shift( @_ );
-    my $no_stoping_file = shift( @_ );
+    my ($this, $no_send, $not_started, $no_stopping_file) = @_;
 
     my $started = $not_started ? undef : $this->is_started;
 
-    $this->{ STOPING } = $started ? 1 : 2;
-
-    if ( !$no_stoping_file ) {
-        open( my $fh, ">$this->{STOPING_R}" );
-        close( $fh );
+    $this->{ STOPPING } = $started ? 1 : 2;
+    
+    if ( !$no_stopping_file ) {
+        $this->touch_file( $this->{STOPPING_R} );
     }
 
     $this->unlock if $started;
@@ -470,6 +476,7 @@ sub stop {
         for ( 1 .. 3 ) { kill( 9, $pid ); }
     }
 
+
     {
         no strict 'refs'; # squash more errors
         close( *{ 'HOLD_PIPE' . $this->{ HOLD_PIPE_X } } );
@@ -481,11 +488,11 @@ sub stop {
 
     sleep( 1 ) if !$started;
 
-    unlink( $this->{ STOPING_R } );
+    unlink( $this->{ STOPPING_R } );
 
     $this->clean_log_dir;
 
-    $this->{ STOPING } = undef;
+    $this->{ STOPPING } = undef;
 
     return 1;
 }
@@ -493,9 +500,9 @@ sub stop {
 
 sub restart {
     my $this = shift;
-
     $this->stop;
     $this->start;
+    return 1;
 }
 
 
@@ -524,15 +531,12 @@ sub read_processR {
 sub save_file_startR {
     my $this = shift;
 
-    open( my $fh, ">$this->{START_R}" );
+    my $pid_r          = win32_double_bs( $this->{ PID_R } );
+    my $process_r      = win32_double_bs( $this->{ PROCESS_R } ); 
+    my $input_r_prefix = win32_double_bs( $this->{ INPUT_R_PREFIX } );
+    my $input_r_suffix = $this->{ INPUT_R_SUFFIX };
 
-    my $process_r = $this->{ PROCESS_R };
-    $process_r =~ s/\\/\\\\/g;
-
-    my $pid_r = $this->{ PID_R };
-    $pid_r =~ s/\\/\\\\/g;
-
-    print $fh qq`
+    my $bridge_code = qq`
       print("Statistics::R - Perl bridge started!") ;
       
       PERLINPUTFILEX = 0 ;
@@ -560,7 +564,7 @@ sub save_file_startR {
         ##print(PERLINPUTFILEX) ;
         cat(PERLINPUTFILEX , "\\n" , file=PERLOUTPUTFILE)
         
-        PERLINPUTFILE <- paste("input.", PERLINPUTFILEX , ".r" , sep="") ;
+        PERLINPUTFILE <- paste("$input_r_prefix", PERLINPUTFILEX , "$input_r_suffix" , sep="") ;
         
         ##print(PERLINPUTFILE) ;
         
@@ -605,204 +609,191 @@ sub save_file_startR {
       
       close(PERLOUTPUTFILE) ;
     `;
-
-    close( $fh );
+    
+    $this->write_file( $this->{START_R}, $bridge_code );
 
     chmod( 0777, $this->{ START_R } );
+    
+    return 1;
 }
 
 
-sub find_file {
-    my $this  = shift;
-    my @files = ref $_[ 0 ] ? @{ shift() } : shift;
-    my @paths = @_;
-
-    for my $path ( @paths ) {
-        for my $file ( @files ) {
-            my $fn = catfile( $path, $file);
-            return $fn if -e $fn && -x $fn;
-        }
-    }
-}
-
-
-sub cat_dir {
+sub clean_log_dir {
     my $this = shift;
+    
+    my @files = (
+        $this->{ START_R }   ,  # start.r
+        $this->{ OUTPUT_R }  ,  # output.log
+        $this->{ PROCESS_R } ,  # process.log
+        $this->{ PID_R }     ,  # R.pid
+        $this->{ LOCK_R }    ,  # lock.pid   
+        # These files are explicitly not deleted. Why?
+        #$this->{ STARTING_R },  # R.starting
+        #$this->{ STOPPING_R },  # R.stopping
+    );
+    
+    push @files, glob win32_space_escape( win32_double_bs( $this->{ INPUT_R_PREFIX }.'*'.$this->{ INPUT_R_SUFFIX } ) );
 
-    my ( $dir, $cut, $r, $f ) = @_;
-    $dir =~ s/\\/\//g;
-
-    my @files;
-
-    my @DIR = $dir;
-    foreach my $DIR ( @DIR ) {
-        my $DH;
-        opendir( $DH, $DIR );
-
-        while ( my $filename = readdir $DH ) {
-            next if $filename eq '.' or $filename eq '..';
-            my $file = "$DIR/$filename";
-            if ( $r && -d $file ) {
-                push( @DIR, $file );
-            }
-            elsif ( !$f || !-d $file ) {
-                $file =~ s/^\Q$dir\E\/?//s if $cut;
-                push( @files, $file );
-            }
+    for my $file (@files) {
+        if (-e $file) {
+            unlink $file or warn "Error: Could not remove file $file\n$!\n";
         }
-
-        closedir( $DH );
     }
-
-    return ( @files );
+    
+    return 1;    
 }
 
 
-sub Linux {
-    my( $this, %args ) = @_;
-
-    $this->{ R_BIN }   = $args{ r_bin }   || $args{ R_bin } || '';
-    $this->{ R_DIR }   = $args{ r_dir }   || $args{ R_dir } || '';
-    $this->{ TMP_DIR } = $args{ tmp_dir } || '';
-
-    if ( !-s $this->{ R_BIN } ) {
-        my @files = qw(R R-project Rproject);
-        ## my @path = (split(":" , $ENV{PATH} || $ENV{Path} || $ENV{path} ) , '/usr/lib/R/bin' , '/usr/lib/R/bin' ) ;
-        # CHANGE MADE BY CTBROWN 2008-06-16
-        # RESPONSE TO RT BUG#23948: bug in Statistics::R
-        my @path = (
-            split( ":", $ENV{ PATH } || $ENV{ Path } || $ENV{ path } ),
-            '/usr/lib/R/bin'
-        );
-
-        my $bin;
-        while ( !$bin && @files ) {
-            $bin = $this->find_file( shift( @files ), @path );
-        }
-
-        if ( !$bin ) {
-            my $path = `which R`;
-            $path =~ s/^\s+//s;
-            $path =~ s/\s+$//s;
-            if ( -e $path && -x $path ) { $bin = $path; }
-        }
-
-        $this->{ R_BIN } = $bin;
-    }
-
-    if ( !$this->{ R_DIR } && $this->{ R_BIN } ) {
-        ( $this->{ R_DIR } )
-            = ( $this->{ R_BIN } =~ /^(.*?)[\\\/]+[^\\\/]+$/s );
-        $this->{ R_DIR } =~ s/\/bin$//;
-    }
-
-    if ( !$this->{ TMP_DIR } ) {
-        foreach my $dir ( qw(/tmp /usr/local/tmp) ) {
-            if ( -d $dir ) { $this->{ TMP_DIR } = $dir; last; }
-        }
-    }
-
-    if ( !-s $this->{ R_BIN } ) {
-        die "Error: Could not find the R binary!\n";
-    }
-    if ( !-d $this->{ R_DIR } ) {
-        die "Error: Could not find the R directory!\n";
-    }
-
-    $this->{ START_CMD } = "$this->{R_BIN} --slave --vanilla ";
-
-    if ( !$args{ log_dir } ) {
-        $args{ log_dir } = catfile( $this->{TMP_DIR}, 'Statistics-R');
-    }
-
-    $this->{ OS } = 'linux';
-
-    $this->pipe( %args );
+sub read_file {
+    # Read content from a file and return it
+    my ($this, $file) = @_;
+    open( my $fh, '<', $file ) or die "Error: Could not read file $file\n$!\n";
+    my $content = join '', <$fh>;
+    $content =~ s/\s//gs;
+    close $fh;
+    return 1;   
 }
 
 
-sub Win32 {
-    my( $this, %args ) = @_;
-
-    $this->{ R_BIN }   = $args{ r_bin }   || $args{ R_bin } || '';
-    $this->{ R_DIR }   = $args{ r_dir }   || $args{ R_dir } || '';
-    $this->{ TMP_DIR } = $args{ tmp_dir } || '';
-
-    if ( !-s $this->{ R_BIN } ) {
-        my $ver_dir = ( $this->cat_dir( "$ENV{ProgramFiles}/R" ) )[ 0 ];
-
-        my $bin = "$ver_dir/bin/Rterm.exe";
-        if ( !-e $bin || !-x $bin ) { $bin = undef; }
-
-        if ( !$bin ) {
-            my @dir = $this->cat_dir( "$ENV{ProgramFiles}/R", undef, 1, 1 );
-            foreach my $dir_i ( @dir ) {
-                if ( $dir_i =~ /\/Rterm\.exe$/ ) { $bin = $dir_i; last; }
-            }
-        }
-
-        if ( !$bin ) {
-            my @files = qw(Rterm.exe);
-            my @path  = (
-                split( ";", $ENV{ PATH } || $ENV{ Path } || $ENV{ path } ) );
-            $bin = $this->find_file( \@files, @path );
-        }
-
-        $this->{ R_BIN } = $bin;
-    }
-
-    if ( !$this->{ R_DIR } && $this->{ R_BIN } ) {
-        ( $this->{ R_DIR } )
-            = ( $this->{ R_BIN } =~ /^(.*?)[\\\/]+[^\\\/]+$/s );
-        $this->{ R_DIR } =~ s/\/bin$//;
-    }
-
-    if ( !$this->{ TMP_DIR } ) {
-        $this->{ TMP_DIR } = $ENV{ TMP } || $ENV{ TEMP };
-        if ( !$this->{ TMP_DIR } ) {
-            foreach
-                my $dir ( qw(c:/tmp c:/temp c:/windows/tmp c:/windows/temp) )
-            {
-                if ( -d $dir ) { $this->{ TMP_DIR } = $dir; last; }
-            }
-        }
-    }
-
-    if ( !-s $this->{ R_BIN } ) {
-        die "Error: Could not find the R binary!\n";
-    }
-    if ( !-d $this->{ R_DIR } ) {
-        die "Error: Could not find the R directory!\n";
-    }
-
-    $this->{ R_BIN }   =~ s/\//\\/g;
-    $this->{ R_DIR }   =~ s/\//\\/g;
-    $this->{ TMP_DIR } =~ s/[\/\\]+/\//g;
-
-    my $exec = $this->{ R_BIN };
-    $exec = "\"$exec\"" if $exec =~ /\s/;
-
-    $this->{ START_CMD } = "$exec --slave --vanilla";
-
-    if ( !$args{ log_dir } ) {
-
-        # $args{log_dir} = "$this->{R_DIR}/Statistics-R" ;
-        # Bug Fix by CTB:  Reponse to RT Bug #17956: Win32: log_dir is not in tmp_dir by default as advertised
-        $args{ log_dir } = catfile( $this->{TMP_DIR}, 'Statistics-R');
-        $args{ log_dir } =~ s/\\+/\//gs;
-    }
-
-    $this->{ OS } = 'win32';
-
-    $this->pipe( %args );
+sub write_file {
+    # Write some content in a file
+    my ($this, $file, $content) = @_;
+    open( my $fh, '>', $file ) or die "Error: Could not write file $file\n$!\n";
+    print $fh "$content";
+    close $fh;
+    return 1;
 }
 
 
+sub touch_file {
+    # Create an empty file
+    my ($this, $file) = @_;
+    open( my $fh, '>', $file ) or die "Error: Could not write file $file\n$!\n";
+    close $fh;
+    return 1;
+}
+
+
+sub win32_path_adjust {
+    # Find potential R directories in the Windows Program Files folder and add
+    # them to the PATH environment variable
+    
+    # Find potential R directories, e.g.  C:\Program Files (x86)\R-2.1\bin
+    #                                 or  C:\Program Files\R\bin\x64
+    my @r_dirs;
+    my @prog_file_dirs   = ($PROGRAMFILES);                # e.g. C:\Program Files (x86)
+    my ($programfiles_2) = ($PROGRAMFILES =~ m/^(.*) \(/); # e.g. C:\Program Files
+    push @prog_file_dirs, $programfiles_2 if not $programfiles_2 eq $PROGRAMFILES;
+    for my $prog_file_dir ( @prog_file_dirs ) {
+        next if not -d $prog_file_dir;
+        my @subdirs;
+        my @globs = ( catfile($prog_file_dir, 'R'), catfile($prog_file_dir, 'R-*') );
+        for my $glob ( @globs ) {
+            $glob = win32_space_escape( win32_double_bs( $glob ) );
+            push @subdirs, glob $glob; # DosGlob
+        }
+        for my $subdir (@subdirs) {
+            my $subdir2 = catfile($subdir, 'bin');
+            if ( -d $subdir2 ) {
+                my $subdir3 = catfile($subdir2, 'x64');
+                if ( -d $subdir3 ) {
+                    push @r_dirs, $subdir3;
+                }
+                push @r_dirs, $subdir2;
+            }
+            push @r_dirs, $subdir;
+        }
+    }
+
+    # Append R directories to PATH (order is important for File::Which)
+    push @PATH, @r_dirs;
+    
+    return 1;
+}
+
+
+sub win32_space_quote {
+    # Quote a path if it contains whitespaces
+    my $path = shift;
+    $path = '"'.$path.'"' if $path =~ /\s/;
+    return $path;
+}
+
+
+sub win32_space_escape {
+    # Escape spaces with a single backslash
+    my $path = shift;
+    $path =~ s/ /\\ /g;
+    return $path;
+}
+
+
+sub win32_double_bs {
+    # Double the backslashes
+    my $path = shift;
+    $path =~ s/\\/\\\\/g;
+    return $path;
+}
+
+
+#sub Linux {
+#    # Find the full path of the R binary on non-Windows systems
+#    my( $this, %args ) = @_;
+#    if ( !-s $this->{ R_BIN } ) {
+#        my @files = qw(R R-project Rproject);
+#        ## my @path = (split(":" , $ENV{PATH} || $ENV{Path} || $ENV{path} ) ,
+#        # '/usr/lib/R/bin' , '/usr/lib/R/bin' ) ;
+#        # CHANGE MADE BY CTBROWN 2008-06-16
+#        # RESPONSE TO RT BUG#23948: bug in Statistics::R
+#        my @path = (
+#            split( ":", $ENV{ PATH } || $ENV{ Path } || $ENV{ path } ),
+#            '/usr/lib/R/bin'
+#        );
+#        my $bin;
+#        while ( !$bin && @files ) {
+#            $bin = $this->find_file( shift( @files ), @path );
+#        }
+#        if ( !$bin ) {
+#            my $path = `which R`;
+#            $path =~ s/^\s+//s;
+#            $path =~ s/\s+$//s;
+#            if ( -e $path && -x $path ) { $bin = $path; }
+#        }
+#        $this->{ R_BIN } = $bin;
+#    }  
+#}
+
+
+#sub Win32 {
+#    # Find the full path of the R binary on Windows systems
+#    my( $this, %args ) = @_;
+#    if ( !-s $this->{ R_BIN } ) {
+#        my $ver_dir = ( $this->cat_dir( catfile($ENV{ProgramFiles},'R') ) )[ 0 ];
+#        my $bin = catfile($ver_dir, 'bin', 'Rterm.exe');
+#        if ( !-e $bin || !-x $bin ) { $bin = undef; }
+#        if ( !$bin ) {
+#            my @dir = $this->cat_dir( catfile($ENV{ProgramFiles},'R'), undef, 1, 1 );
+#            foreach my $dir_i ( @dir ) {
+#                if ( $dir_i =~ /\/Rterm\.exe$/ ) { $bin = $dir_i; last; }
+#            }
+#        }
+#        if ( !$bin ) {
+#            my @files = qw(Rterm.exe);
+#            my @path  = (
+#                split( ";", $ENV{ PATH } || $ENV{ Path } || $ENV{ path } ) );
+#            $bin = $this->find_file( \@files, @path );
+#        }
+#        $this->{ R_BIN } = $bin;
+#    }
+#    if ( !-s $this->{ R_BIN } ) {
+#        die "Error: Could not find the R binary!\n";
+#    }
+#    $this->{ R_BIN } = '"'.$this->{ R_BIN }.'"' if $this->{ R_BIN } =~ /\s/;
+#}
+
+
 sub DESTROY {
     my $this = shift;
-
-    return unless $this->{ OS };
-
     $this->unlock;
     $this->stop if !$this->{ START_SHARED };
 }
@@ -814,11 +805,11 @@ __END__
 
 =head1 NAME
 
-Statistics::R::Bridge - Implements a communication bridge between Perl and R.
+Statistics::R::Bridge - A communication bridge between Perl and R.
 
 =head1 DESCRIPTION
 
-This will implements a communication bridge between Perl and R (R project for
+This module implements a communication bridge between Perl and R (R project for
 statistical computing: L<http://www.r-project.org/>) in different architectures
 and OS.
 
